@@ -1,6 +1,6 @@
-;;; semantic/analyze/debug.el --- Debug the analyzer
+;;; semantic/analyze/debug.el --- Debug the analyzer  -*- lexical-binding: t; -*-
 
-;;; Copyright (C) 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
+;; Copyright (C) 2008-2022 Free Software Foundation, Inc.
 
 ;; Author: Eric M. Ludlam <zappo@gnu.org>
 
@@ -17,7 +17,7 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
+;; along with GNU Emacs.  If not, see <https://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 ;;
@@ -28,6 +28,7 @@
 (require 'semantic/analyze)
 (require 'semantic/analyze/complete)
 (require 'semantic/db-typecache)
+(require 'pulse)
 
 ;; For semantic-find-tags-by-class:
 (eval-when-compile (require 'semantic/find))
@@ -70,7 +71,7 @@
       (let ((pentry (nth idx pf))
 	    (ptentry (nth idx pft)))
 	(if (or (stringp pentry) (not ptentry))
-	    ;; Found someting ok.  stop
+	    ;; Found something ok.  Stop.
 	    (setq stop t)
 	  (setq idx (1+ idx)))))
     ;; We found the first non-tag entry.  What is the situation?
@@ -108,11 +109,11 @@ Argument COMP are possible completions here."
 	(condition-case err
 	    (with-current-buffer origbuf
 	      (let* ((position (or (cdr-safe (oref ctxt bounds)) (point)))
-		     (prefixtypes nil) ; Used as type return
+		     ;; (semantic--prefixtypes nil) ; Used as type return
 		     (scope (semantic-calculate-scope position))
 		     )
 		(semantic-analyze-find-tag-sequence
-		 (list prefix "") scope 'prefixtypes)
+		 (list prefix "") scope) ;; 'semantic--prefixtypes
 		)
 	      )
 	  (error (setq finderr err)))
@@ -148,7 +149,7 @@ path was setup incorrectly.\n")
     (semantic-analyzer-debug-add-buttons)
     ))
 
-(defun semantic-analyzer-debug-missing-datatype (ctxt idx comp)
+(defun semantic-analyzer-debug-missing-datatype (ctxt idx _comp)
   "Debug why we can't find a datatype entry for CTXT prefix at IDX.
 Argument COMP are possible completions here."
   (let* ((prefixitem (nth idx (oref ctxt prefix)))
@@ -405,18 +406,19 @@ or implementing a version specific to ")
 
     (princ "\n\nInclude Path Summary:")
     (when edeobj
-	(princ "\n\nThis file's project include search is handled by the EDE object:\n")
+	(princ (substitute-command-keys
+		"\n\nThis file's project include search is handled by the EDE object:\n"))
 	(princ "  Buffer Target:  ")
-	(princ (object-print edeobj))
+	(princ (cl-prin1-to-string edeobj))
 	(princ "\n")
 	(when (not (eq edeobj edeproj))
 	  (princ "  Buffer Project: ")
-	  (princ (object-print edeproj))
+	  (princ (cl-prin1-to-string edeproj))
 	  (princ "\n"))
 	(when edeproj
 	  (let ((loc (ede-get-locator-object edeproj)))
 	    (princ "  Backup Locator: ")
-	    (princ (object-print loc))
+	    (princ (cl-prin1-to-string loc))
 	    (princ "\n")))
 	)
 
@@ -443,7 +445,7 @@ or implementing a version specific to ")
 			     (semanticdb-file-table-object fileinner t))))
 	  (cond ((not fileinner)
 		 (setq unknown (1+ unknown)))
-		((number-or-marker-p (oref tableinner pointmax))
+		((and tableinner (number-or-marker-p (oref tableinner pointmax)))
 		 (setq ok (1+ ok)))
 		(t
 		 (setq unparsed (1+ unparsed))))))
@@ -463,12 +465,12 @@ or implementing a version specific to ")
 
 	  (princ "\nYou can fix the include path for ")
 	  (princ (symbol-name (oref table major-mode)))
-	  (princ " by using this function:
+	  (princ (substitute-command-keys " by using this function:
 
-M-x semantic-customize-system-include-path RET
+\\[semantic-customize-system-include-path]
 
 which customizes the mode specific variable for the mode-local
-variable `semantic-dependency-system-include-path'.")
+variable `semantic-dependency-system-include-path'."))
 	  )
 
       (princ "\n No unknown includes.\n"))
@@ -477,7 +479,7 @@ variable `semantic-dependency-system-include-path'.")
 (defun semantic-analyzer-debug-describe-scope (ctxt &optional classconstraint)
   "Describe the scope in CTXT for finding a global symbol.
 Optional argument CLASSCONSTRAINT says to output to tags of that class."
-  (let* ((scope (oref ctxt :scope))
+  (let* ((scope (oref ctxt scope))
 	 (parents (oref scope parents))
 	 (cc (or classconstraint (oref ctxt prefixclass)))
 	 )
@@ -512,7 +514,7 @@ Optional argument CLASSCONSTRAINT says to output to tags of that class."
   )
 
 (defun semantic-analyzer-debug-global-miss-text (name-in)
-  "Use 'princ' to show text describing not finding symbol NAME-IN.
+  "Use `princ' to show text describing not finding symbol NAME-IN.
 NAME is the name of the unfound symbol."
   (let ((name (cond ((stringp name-in)
 		     name-in)
@@ -557,19 +559,19 @@ PARENT is a possible parent (by nesting) tag."
 			 'mouse-face 'custom-button-pressed-face
 			 'tag tag
 			 'action
-			 `(lambda (button)
-			    (let ((buff nil)
-				  (pnt nil))
-			      (save-excursion
-				(semantic-go-to-tag
-				 (button-get button 'tag))
-				(setq buff (current-buffer))
-				(setq pnt (point)))
-			      (if (get-buffer-window buff)
-				  (select-window (get-buffer-window buff))
-				(pop-to-buffer buff t))
-			      (goto-char pnt)
-			      (pulse-line-hook-function)))
+			 (lambda (button)
+			   (let ((buff nil)
+				 (pnt nil))
+			     (save-excursion
+			       (semantic-go-to-tag
+				(button-get button 'tag))
+			       (setq buff (current-buffer))
+			       (setq pnt (point)))
+			     (if (get-buffer-window buff)
+				 (select-window (get-buffer-window buff))
+			       (pop-to-buffer buff t))
+			     (goto-char pnt)
+			     (pulse-line-hook-function)))
 			 ))
       (princ "\"")
       (princ str)
@@ -588,22 +590,23 @@ Look for key expressions, and add push-buttons near them."
     (with-current-buffer "*Help*"
       (let ((inhibit-read-only t))
 	(goto-char (point-min))
-	(set (make-local-variable 'semantic-analyzer-debug-orig) orig-buffer)
+        (setq-local semantic-analyzer-debug-orig orig-buffer)
 	;; First, add do-in buttons to recommendations.
 	(while (re-search-forward "^\\s-*M-x \\(\\(\\w\\|\\s_\\)+\\) " nil t)
-	  (let ((fcn (match-string 1)))
-	    (when (not (fboundp (intern-soft fcn)))
+	  (let* ((fcn (match-string 1))
+	         (fsym (intern-soft fcn)))
+	    (when (not (fboundp fsym))
 	      (error "Help Err: Can't find %s" fcn))
 	    (end-of-line)
 	    (insert "   ")
 	    (insert-button "[ Do It ]"
 			   'mouse-face 'custom-button-pressed-face
 			   'do-fcn fcn
-			   'action `(lambda (arg)
-				      (let ((M semantic-analyzer-debug-orig))
-					(set-buffer (marker-buffer M))
-					(goto-char M))
-				      (call-interactively (quote ,(intern-soft fcn))))))))
+			   'action (lambda (_arg)
+				     (let ((M semantic-analyzer-debug-orig))
+				       (set-buffer (marker-buffer M))
+				       (goto-char M))
+				     (call-interactively fsym))))))
       ;; Do something else?
       ;; Clean up the mess
       (set-buffer-modified-p nil))))
