@@ -1,7 +1,7 @@
-;;; page-ext.el --- extended page handling commands
+;;; page-ext.el --- extended page handling commands  -*- lexical-binding:t -*-
 
-;; Copyright (C) 1990, 1991, 1993, 1994, 2001, 2002, 2003, 2004, 2005, 2006,
-;;   2007, 2008, 2009, 2010, 2011, 2012  Free Software Foundation, Inc.
+;; Copyright (C) 1990-1991, 1993-1994, 2001-2022 Free Software
+;; Foundation, Inc.
 
 ;; Author: Robert J. Chassell <bob@gnu.org>
 ;; (according to ack.texi)
@@ -20,7 +20,7 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
+;; along with GNU Emacs.  If not, see <https://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
@@ -47,12 +47,12 @@
 
 ;; New page handling commands:
 
-;;     next-page                        C-x C-p C-n
-;;     previous-page                    C-x C-p C-p
-;;     search-pages                     C-x C-p C-s
-;;     add-new-page                     C-x C-p C-a
-;;     sort-pages-buffer                C-x C-p s
-;;     set-page-delimiter               C-x C-p C-l
+;;     pages-next-page                  C-x C-p C-n
+;;     pages-previous-page              C-x C-p C-p
+;;     pages-search                     C-x C-p C-s
+;;     pages-add-new-page               C-x C-p C-a
+;;     pages-sort-buffer                C-x C-p s
+;;     pages-set-delimiter              C-x C-p C-l
 ;;     pages-directory                  C-x C-p C-d
 ;;     pages-directory-for-addresses    C-x C-p d
 ;;        pages-directory-goto          C-c C-c
@@ -156,11 +156,11 @@
 ;;     George Lakoff
 ;;     OBI (On line text collection.)
 
-;; The `C-x C-p s' (sort-pages-buffer) command sorts the entries in the
+;; The `C-x C-p s' (pages-sort-buffer) command sorts the entries in the
 ;; buffer alphabetically.
 
-;; You may use any of the page commands, including the `next-page',
-;; `previous-page', `add-new-page', `mark-page', and `search-pages'
+;; You may use any of the page commands, including the `pages-next-page',
+;; `pages-previous-page', `pages-add-new-page', `mark-page', and `pages-search'
 ;; commands.
 
 ;; You may use either the `C-x C-p d' (pages-directory-for-addresses)
@@ -171,7 +171,7 @@
 ;; and type `C-c C-c' (pages-directory-goto) to go to the entry to
 ;; which it refers in the pages buffer.
 
-;; You can type `C-c C-p C-a' (add-new-page) to add a new entry in the
+;; You can type `C-c C-p C-a' (pages-add-new-page) to add a new entry in the
 ;; pages buffer or address file.  This is the same command you use to
 ;; add a new entry when you are in the pages buffer or address file.
 
@@ -242,117 +242,116 @@
 
 
 (defcustom pages-directory-buffer-narrowing-p t
-  "*If non-nil, `pages-directory-goto' narrows pages buffer to entry."
-  :type 'boolean
-  :group 'pages)
+  "If non-nil, `pages-directory-goto' narrows pages buffer to entry."
+  :type 'boolean)
 
 (defcustom pages-directory-for-adding-page-narrowing-p t
-  "*If non-nil, `add-new-page' narrows page buffer to new entry."
-  :type 'boolean
-  :group 'pages)
+  "If non-nil, `add-new-page' narrows page buffer to new entry."
+  :type 'boolean)
 
 (defcustom pages-directory-for-adding-new-page-before-current-page-p t
-  "*If non-nil, `add-new-page' inserts new page before current page."
-  :type 'boolean
-  :group 'pages)
+  "If non-nil, `add-new-page' inserts new page before current page."
+  :type 'boolean)
 
 
 ;;; Addresses related variables
 
 (defcustom pages-addresses-file-name "~/addresses"
-  "*Standard name for file of addresses. Entries separated by page-delimiter.
+  "Standard name for file of addresses.  Entries separated by `page-delimiter'.
 Used by `pages-directory-for-addresses' function."
-  :type 'file
-  :group 'pages)
+  :type 'file)
 
 (defcustom pages-directory-for-addresses-goto-narrowing-p t
-  "*If non-nil, `pages-directory-goto' narrows addresses buffer to entry."
-  :type 'boolean
-  :group 'pages)
+  "If non-nil, `pages-directory-goto' narrows addresses buffer to entry."
+  :type 'boolean)
 
 (defcustom pages-directory-for-addresses-buffer-keep-windows-p t
-  "*If nil, `pages-directory-for-addresses' deletes other windows."
-  :type 'boolean
-  :group 'pages)
+  "If nil, `pages-directory-for-addresses' deletes other windows."
+  :type 'boolean)
 
 (defcustom pages-directory-for-adding-addresses-narrowing-p t
-  "*If non-nil, `add-new-page' narrows addresses buffer to new entry."
-  :type 'boolean
-  :group 'pages)
+  "If non-nil, `add-new-page' narrows addresses buffer to new entry."
+  :type 'boolean)
 
 
 ;;; Key bindings for page handling functions
 
-(global-unset-key "\C-x\C-p")
-
-(defvar ctl-x-ctl-p-map (make-sparse-keymap)
+(defvar pages--ctl-x-ctl-p-map
+  (let ((map (make-sparse-keymap)))
+    (define-key map "\C-n" #'pages-next-page)
+    (define-key map "\C-p" #'pages-previous-page)
+    (define-key map "\C-a" #'pages-add-new-page)
+    (define-key map "\C-m" #'mark-page)
+    (define-key map "\C-s" #'pages-search)
+    (define-key map "s"    #'pages-sort-buffer)
+    (define-key map "\C-l" #'pages-set-delimiter)
+    (define-key map "\C-d" #'pages-directory)
+    (define-key map "d"    #'pages-directory-for-addresses)
+    map)
   "Keymap for subcommands of C-x C-p, which are for page handling.")
 
-(define-key ctl-x-map "\C-p" 'ctl-x-ctl-p-prefix)
-(fset 'ctl-x-ctl-p-prefix ctl-x-ctl-p-map)
-
-(define-key ctl-x-ctl-p-map "\C-n" 'next-page)
-(define-key ctl-x-ctl-p-map "\C-p" 'previous-page)
-(define-key ctl-x-ctl-p-map "\C-a" 'add-new-page)
-(define-key ctl-x-ctl-p-map "\C-m" 'mark-page)
-(define-key ctl-x-ctl-p-map "\C-s" 'search-pages)
-(define-key ctl-x-ctl-p-map "s"    'sort-pages-buffer)
-(define-key ctl-x-ctl-p-map "\C-l" 'set-page-delimiter)
-(define-key ctl-x-ctl-p-map "\C-d" 'pages-directory)
-(define-key ctl-x-ctl-p-map "d"    'pages-directory-for-addresses)
+;; FIXME: Merely loading a package shouldn't have this kind of side-effects!
+(global-unset-key "\C-x\C-p")
+(define-key ctl-x-map "\C-p" #'pages-ctl-x-ctl-p-prefix)
+(define-obsolete-function-alias 'ctl-x-ctl-p-prefix #'pages-ctl-x-ctl-p-prefix "27.1")
+(defalias 'pages-ctl-x-ctl-p-prefix pages--ctl-x-ctl-p-map)
 
 
 ;;; Page movement function definitions
 
-(defun next-page (&optional count)
+(define-obsolete-function-alias 'next-page #'pages-next-page "27.1")
+(defun pages-next-page (&optional count)
   "Move to the next page bounded by the `page-delimiter' variable.
 With arg (prefix if interactive), move that many pages."
   (interactive "p")
   (or count (setq count 1))
   (widen)
   ;; Cannot use forward-page because of problems at page boundaries.
-  (while (and (> count 0) (not (eobp)))
-    (if (re-search-forward page-delimiter nil t)
-        nil
-      (goto-char (point-max)))
-    (setq count (1- count)))
-  ;; If COUNT is negative, we want to go back -COUNT + 1 page boundaries.
-  ;; The first page boundary we reach is the top of the current page,
-  ;; which doesn't count.
-  (while (and (< count 1) (not (bobp)))
-    (if (re-search-backward page-delimiter nil t)
-	(goto-char (match-beginning 0))
-      (goto-char (point-min)))
-    (setq count (1+ count)))
+  (if (>= count 0)
+      (while (and (> count 0) (not (eobp)))
+        (if (re-search-forward page-delimiter nil t)
+            nil
+          (goto-char (point-max)))
+        (setq count (1- count)))
+    ;; If COUNT is negative, we want to go back -COUNT + 1 page boundaries.
+    ;; The first page boundary we reach is the top of the current page,
+    ;; which doesn't count.
+    (while (and (< count 1) (not (bobp)))
+      (if (re-search-backward page-delimiter nil t)
+          (when (= count 0)
+            (goto-char (match-end 0)))
+        (goto-char (point-min)))
+      (setq count (1+ count))))
   (narrow-to-page)
   (goto-char (point-min))
   (recenter 0))
 
-(defun previous-page (&optional count)
+(define-obsolete-function-alias 'previous-page #'pages-previous-page "27.1")
+(defun pages-previous-page (&optional count)
   "Move to the previous page bounded by the `page-delimiter' variable.
 With arg (prefix if interactive), move that many pages."
   (interactive "p")
   (or count (setq count 1))
-  (next-page (- count)))
+  (pages-next-page (- count)))
 
 
 ;;; Adding and searching pages
 
-(defun add-new-page (header-line)
+(define-obsolete-function-alias 'add-new-page #'pages-add-new-page "27.1")
+(defun pages-add-new-page (header-line)
   "Insert new page.  Prompt for header line.
 
 If point is in the pages directory buffer, insert the new page in the
 buffer associated with the directory.
 
 Insert the new page just before current page if
-  pages-directory-for-adding-new-page-before-current-page-p  variable
-is non-nil.  Else insert at exact location of point.
+`pages-directory-for-adding-new-page-before-current-page-p' is non-nil.
+Else insert at exact location of point.
 
-Narrow to new page if
-  pages-directory-for-adding-page-narrowing-p variable
-is non-nil.
+Narrow to new page if `pages-directory-for-adding-page-narrowing-p' is
+non-nil.
 
-Page begins with a `^L' as the default page-delimiter.
+Page begins with a `^L' as the default `page-delimiter'.
 Use \\[set-page-delimiter] to change the page-delimiter.
 Point is left in the body of page."
   (interactive "sHeader line: ")
@@ -392,12 +391,13 @@ Point is left in the body of page."
 (defvar pages-last-search nil
   "Value of last regexp searched for.  Initially, nil.")
 
-(defun search-pages (regexp)
+(define-obsolete-function-alias 'search-pages #'pages-search "27.1")
+(defun pages-search (regexp)
   "Search for REGEXP, starting from point, and narrow to page it is in."
   (interactive (list
                 (read-string
-                 (format "Search for `%s' (end with RET): "
-                         (or pages-last-search "regexp")))))
+                 (format-message "Search for `%s' (end with RET): "
+				 (or pages-last-search "regexp")))))
   (if (equal regexp "")
       (setq regexp pages-last-search)
     (setq pages-last-search regexp))
@@ -408,17 +408,16 @@ Point is left in the body of page."
 
 ;;; Sorting pages
 
-(autoload 'sort-subr "sort" "Primary function for sorting." t nil)
-
-(defun sort-pages-in-region (reverse beg end)
+(define-obsolete-function-alias 'sort-pages-in-region #'pages-sort-region "27.1")
+(defun pages-sort-region (reverse beg end)
   "Sort pages in region alphabetically.  Prefix arg means reverse order.
 
 Called from a program, there are three arguments:
 REVERSE (non-nil means reverse order), BEG and END (region to sort)."
 
-;;; This sort function handles ends of pages differently than
-;;; `sort-pages' and works better with lists of addresses and similar
-;;; files.
+  ;; This sort function handles ends of pages differently than
+  ;; `sort-pages' and works better with lists of addresses and similar
+  ;; files.
 
   (interactive "P\nr")
   (save-restriction
@@ -430,31 +429,31 @@ REVERSE (non-nil means reverse order), BEG and END (region to sort)."
                ;; NEXTRECFUN is called with point at the end of the
                ;; previous record. It moves point to the start of the
                ;; next record.
-	       (function (lambda ()
-                           (re-search-forward page-delimiter nil t)
-                           (skip-chars-forward " \t\n")
-                           ))
+               (lambda ()
+                 (re-search-forward page-delimiter nil t)
+                 (skip-chars-forward " \t\n"))
 
-               ;; ENDRECFUN is is called with point within the record.
+               ;; ENDRECFUN is called with point within the record.
                ;; It should move point to the end of the record.
-	       (function (lambda ()
-                           (if (re-search-forward
-                                page-delimiter
-                                nil
-                                t)
-                               (goto-char (match-beginning 0))
-                             (goto-char (point-max))))))))
+               (lambda ()
+                 (if (re-search-forward
+                      page-delimiter
+                      nil
+                      t)
+                     (goto-char (match-beginning 0))
+                   (goto-char (point-max)))))))
 
-(defun sort-pages-buffer (&optional reverse)
+(define-obsolete-function-alias 'sort-pages-buffer #'pages-sort-buffer "27.1")
+(defun pages-sort-buffer (&optional reverse)
   "Sort pages alphabetically in buffer.  Prefix arg means reverse order.
-\(Non-nil arg if not interactive.\)"
+\(Non-nil arg if not interactive.)"
 
   (interactive "P")
   (or reverse (setq reverse nil))
   (widen)
   (let ((beginning (point-min))
         (end (point-max)))
-    (sort-pages-in-region reverse beginning end)))
+    (pages-sort-region reverse beginning end)))
 
 
 ;;; Pages directory ancillary definitions
@@ -462,32 +461,35 @@ REVERSE (non-nil means reverse order), BEG and END (region to sort)."
 (defvar pages-directory-previous-regexp nil
   "Value of previous regexp used by `pages-directory'.
 \(This regular expression may be used to select only those pages that
-contain matches to the regexp.\)")
+contain matches to the regexp.)")
 
-(defvar pages-buffer nil
+(defvar-local pages-buffer nil
   "The buffer for which the pages-directory function creates the directory.")
 
 (defvar pages-directory-prefix "*Directory for:"
   "Prefix of name of temporary buffer for pages-directory.")
 
-(defvar pages-pos-list nil
+(defvar-local pages-pos-list nil
   "List containing the positions of the pages in the pages-buffer.")
 
 (defvar pages-target-buffer)
 
+(define-obsolete-variable-alias 'pages-directory-map
+  'pages-directory-mode-map "26.1")
 (defvar pages-directory-mode-map
   (let ((map (make-sparse-keymap)))
-    (define-key map "\C-c\C-c" 'pages-directory-goto)
-    (define-key map "\C-c\C-p\C-a" 'add-new-page)
-    (define-key map [mouse-2] 'pages-directory-goto-with-mouse)
+    (define-key map "\C-c\C-c"     #'pages-directory-goto)
+    (define-key map "\C-m"         #'pages-directory-goto)
+    (define-key map "\C-c\C-p\C-a" #'pages-add-new-page)
+    (define-key map [mouse-2]      #'pages-directory-goto)
     map)
   "Keymap for the pages-directory-buffer.")
-(defvaralias 'pages-directory-map 'pages-directory-mode-map)
 
-(defvar original-page-delimiter "^\f"
+(defvar pages-original-delimiter "^\f"
   "Default page delimiter.")
 
-(defun set-page-delimiter (regexp reset-p)
+(define-obsolete-function-alias 'set-page-delimiter #'pages-set-delimiter "27.1")
+(defun pages-set-delimiter (regexp reset-p &optional interactively)
   "Set buffer local value of page-delimiter to REGEXP.
 Called interactively with a prefix argument, reset `page-delimiter' to
 its original value.
@@ -497,26 +499,26 @@ resets the page-delimiter to the original value."
 
   (interactive
    (if current-prefix-arg
-       (list original-page-delimiter "^\f")
-     (list (read-string "Set page-delimiter to regexp: " page-delimiter)
-           nil)))
-  (make-local-variable 'original-page-delimiter)
-  (make-local-variable 'page-delimiter)
-  (setq original-page-delimiter
-        (or original-page-delimiter page-delimiter))
-  (if (not reset-p)
-      (setq page-delimiter regexp)
-    (setq page-delimiter original-page-delimiter))
-  (if (called-interactively-p 'interactive)
+       (list pages-original-delimiter t t)
+     (list (read-regexp "Set page-delimiter to regexp: " page-delimiter)
+           nil t)))
+  (setq-local pages-original-delimiter
+              (or pages-original-delimiter page-delimiter))
+  (setq-local page-delimiter
+              (if (not reset-p) regexp pages-original-delimiter))
+  (if interactively
       (message "The value of `page-delimiter' is now: %s" page-delimiter)))
 
 
 ;;; Pages directory main definitions
 
+(defvar pages-buffer-original-position)
+(defvar pages-buffer-original-page)
+
 (defun pages-directory
   (pages-list-all-headers-p count-lines-p &optional regexp)
   "Display a directory of the page headers in a temporary buffer.
-A header is the first non-blank line after the page-delimiter.
+A header is the first non-blank line after the `page-delimiter'.
 \\[pages-directory-mode]
 You may move point to one of the lines in the temporary buffer,
 then use \\<pages-directory-goto> to go to the same line in the pages buffer.
@@ -550,16 +552,18 @@ directory for only the accessible portion of the buffer."
           (list nil
                 nil
                 (read-string
-                 (format "Select according to `%s' (end with RET): "
-                         (or pages-directory-previous-regexp "regexp")))))
+                 (format-message
+		  "Select according to `%s' (end with RET): "
+		  (or pages-directory-previous-regexp "regexp")))))
          ((> (prefix-numeric-value current-prefix-arg) 0)
           (list t t nil))
          ((< (prefix-numeric-value current-prefix-arg) 0)
           (list nil
                 t
                 (read-string
-                 (format "Select according to `%s' (end with RET): "
-                         (or pages-directory-previous-regexp "regexp")))))))
+                 (format-message
+		  "Select according to `%s' (end with RET): "
+		  (or pages-directory-previous-regexp "regexp")))))))
 
   (if (equal regexp "")
       (setq regexp pages-directory-previous-regexp)
@@ -572,7 +576,6 @@ directory for only the accessible portion of the buffer."
   (let ((pages-target-buffer (current-buffer))
         (pages-directory-buffer
 	 (concat pages-directory-prefix " " (buffer-name)))
-        (linenum 1)
         (pages-buffer-original-position (point))
         (pages-buffer-original-page 0))
 
@@ -582,6 +585,7 @@ directory for only the accessible portion of the buffer."
     (with-output-to-temp-buffer pages-directory-buffer
       (with-current-buffer standard-output
         (pages-directory-mode)
+        (setq buffer-read-only nil)
         (insert
          "==== Pages Directory: use `C-c C-c' to go to page under cursor. ====" ?\n)
         (setq pages-buffer pages-target-buffer)
@@ -630,6 +634,7 @@ directory for only the accessible portion of the buffer."
                 )))))
 
       (set-buffer standard-output)
+      (setq buffer-read-only t)
       ;; Put positions in increasing order to go with buffer.
       (setq pages-pos-list (nreverse pages-pos-list))
       (if (called-interactively-p 'interactive)
@@ -641,10 +646,6 @@ directory for only the accessible portion of the buffer."
                       1
                     pages-buffer-original-page))))
 
-(defvar pages-buffer-original-position)
-(defvar pages-buffer-original-page)
-(defvar pages-buffer-original-page)
-
 (defun pages-copy-header-and-position (count-lines-p)
   "Copy page header and its position to the Pages Directory.
 Only arg non-nil, count lines in page and insert before header.
@@ -671,7 +672,7 @@ Used by `pages-directory' function."
       (setq position (make-marker))
       (set-marker position (point))
       (let ((start (point))
-            (end (save-excursion (end-of-line) (point)))
+            (end (line-end-position))
 	    inserted-at)
         ;; change to directory buffer
         (set-buffer standard-output)
@@ -693,27 +694,18 @@ Used by `pages-directory' function."
       (terpri))
     (end-of-line 1)))
 
-(defun pages-directory-mode ()
+(define-derived-mode pages-directory-mode special-mode "Pages-Directory"
   "Mode for handling the pages-directory buffer.
 
 Move point to one of the lines in this buffer, then use \\[pages-directory-goto] to go
 to the same line in the pages buffer."
+  )
 
-  (kill-all-local-variables)
-  (use-local-map pages-directory-mode-map)
-  (setq major-mode 'pages-directory-mode)
-  (setq mode-name "Pages-Directory")
-  (make-local-variable 'pages-buffer)
-  (make-local-variable 'pages-pos-list)
-  (make-local-variable 'pages-directory-buffer-narrowing-p)
-  (run-mode-hooks 'pages-directory-mode-hook))
-
-(defun pages-directory-goto ()
+(defun pages-directory-goto (&optional event)
   "Go to the corresponding line in the pages buffer."
-
-;;; This function is mostly a copy of `occur-mode-goto-occurrence'
-
-  (interactive)
+  ;; This function is mostly a copy of `occur-mode-goto-occurrence'
+  (interactive (list last-nonmenu-event))
+  (if event (mouse-set-point event))
   (if (or (not pages-buffer)
 	  (not (buffer-name pages-buffer)))
       (progn
@@ -727,18 +719,13 @@ to the same line in the pages buffer."
          (narrowing-p  pages-directory-buffer-narrowing-p))
     (pop-to-buffer pages-buffer)
     (widen)
-    (if end-of-directory-p
-        (goto-char (point-max))
-      (goto-char (marker-position pos)))
+    (goto-char (if end-of-directory-p
+                   (point-max)
+                 (marker-position pos)))
     (if narrowing-p (narrow-to-page))))
 
-(defun pages-directory-goto-with-mouse  (event)
-  "Go to the corresponding line under the mouse pointer in the pages buffer."
-  (interactive "e")
-  (with-current-buffer (window-buffer (posn-window (event-end event)))
-    (save-excursion
-      (goto-char (posn-point (event-end event)))
-      (pages-directory-goto))))
+(define-obsolete-function-alias 'pages-directory-goto-with-mouse
+  #'pages-directory-goto "26.1")
 
 ;;; The `pages-directory-for-addresses' function and ancillary code
 
@@ -752,10 +739,10 @@ Move point to one of the lines in the displayed directory,
 then use \\[pages-directory-goto] to go to the same line
 in the addresses buffer.
 
-If    pages-directory-for-addresses-goto-narrowing-p    is non-nil,
+If `pages-directory-for-addresses-goto-narrowing-p' is non-nil,
 `pages-directory-goto' narrows addresses buffer to entry.
 
-If    pages-directory-for-addresses-buffer-keep-windows-p     is nil,
+If `pages-directory-for-addresses-buffer-keep-windows-p' is nil,
 this command deletes other windows when it displays the addresses
 directory."
 
@@ -777,13 +764,13 @@ directory."
         ;; by RJC, 2006 Jun 11: including this causes failure; it results in
         ;;  the message "Buffer in which pages were found is deleted"
         ;;        (pages-directory-address-mode)
-        (setq pages-directory-buffer-narrowing-p
-              pages-directory-for-addresses-goto-narrowing-p)
+        (setq-local pages-directory-buffer-narrowing-p
+                    pages-directory-for-addresses-goto-narrowing-p)
         (or pages-directory-for-addresses-buffer-keep-windows-p
             (delete-other-windows))
         (save-excursion
           (goto-char (point-min))
-          (delete-region (point) (save-excursion (end-of-line) (point)))
+          (delete-region (point) (line-end-position))
           (insert
            "=== Address List Directory: use `C-c C-c' to go to page under cursor. ===")
           (set-buffer-modified-p nil)
@@ -801,5 +788,4 @@ to the same line in the pages buffer."
 
 (provide 'page-ext)
 
-;; arch-tag: 2f311550-c6e0-4458-9c12-7f039c058bdb
 ;;; page-ext.el ends here
