@@ -1,6 +1,6 @@
-;;; view.el --- peruse file or buffer without editing
+;;; view.el --- peruse file or buffer without editing  -*- lexical-binding: t -*-
 
-;; Copyright (C) 1985, 1989, 1994-1995, 1997, 2000-2019 Free Software
+;; Copyright (C) 1985, 1989, 1994-1995, 1997, 2000-2022 Free Software
 ;; Foundation, Inc.
 
 ;; Author: K. Shane Hartman
@@ -26,9 +26,9 @@
 
 ;; This package provides the `view' minor mode documented in the Emacs
 ;; user's manual.
-;; View mode entry and exit is done through the functions view-mode-enter
-;; and view-mode-exit.  Use these functions to enter or exit view-mode from
-;; emacs lisp programs.
+;; View mode entry and exit is done through the functions `view-mode-enter'
+;; and `view-mode-exit'.  Use these functions to enter or exit `view-mode' from
+;; Emacs Lisp programs.
 ;; We use both view- and View- as prefix for symbols.  View- is used as
 ;; prefix for commands that have a key binding.  view- is used for commands
 ;; without key binding.  The purpose of this is to make it easier for a
@@ -36,11 +36,11 @@
 
 ;;; Suggested key bindings:
 ;;
-;; (define-key ctl-x-4-map "v" 'view-file-other-window)  ; ^x4v
-;; (define-key ctl-x-5-map "v" 'view-file-other-frame)  ; ^x5v
+;; (define-key ctl-x-4-map "v" #'view-file-other-window)  ; ^x4v
+;; (define-key ctl-x-5-map "v" #'view-file-other-frame)   ; ^x5v
 ;;
-;; You could also bind view-file, view-buffer, view-buffer-other-window and
-;; view-buffer-other-frame to keys.
+;; You could also bind `view-file', `view-buffer', `view-buffer-other-window' and
+;; `view-buffer-other-frame' to keys.
 
 ;;; Code:
 
@@ -51,31 +51,27 @@
   :group 'text)
 
 (defcustom view-highlight-face 'highlight
-   "The face used for highlighting the match found by View mode search."
-   :type 'face
-   :group 'view)
+  "The face used for highlighting the match found by View mode search."
+  :type 'face)
 
 (defcustom view-scroll-auto-exit nil
   "Non-nil means scrolling past the end of buffer exits View mode.
 A value of nil means attempting to scroll past the end of the buffer,
 only rings the bell and gives a message on how to leave."
-  :type 'boolean
-  :group 'view)
+  :type 'boolean)
 
 (defcustom view-try-extend-at-buffer-end nil
  "Non-nil means try to load more of file when reaching end of buffer.
 This variable is mainly intended to be temporarily set to non-nil by
-the F command in view-mode, but you can set it to t if you want the action
+the F command in `view-mode', but you can set it to t if you want the action
 for all scroll commands in view mode."
-  :type 'boolean
-  :group 'view)
+  :type 'boolean)
 
 ;;;###autoload
 (defcustom view-remove-frame-by-deleting t
   "Determine how View mode removes a frame no longer needed.
 If nil, make an icon of the frame.  If non-nil, delete the frame."
   :type 'boolean
-  :group 'view
   :version "23.1")
 
 (defcustom view-exits-all-viewing-windows nil
@@ -84,48 +80,40 @@ Commands that restore windows when finished viewing a buffer,
 apply to all windows that display the buffer and have restore
 information.  If `view-exits-all-viewing-windows' is nil, only
 the selected window is considered for restoring."
-  :type 'boolean
-  :group 'view)
+  :type 'boolean)
 
 (defcustom view-inhibit-help-message nil
-  "Non-nil inhibits the help message shown upon entering View mode."
+  "Non-nil inhibits the help message shown upon entering View mode.
+This setting takes effect only when View mode is entered via an
+interactive command; otherwise the help message is not shown."
   :type 'boolean
-  :group 'view
   :version "22.1")
 
 ;;;###autoload
-(defvar view-mode nil
+(defvar-local view-mode nil
   "Non-nil if View mode is enabled.
 Don't change this variable directly, you must change it by one of the
 functions that enable or disable view mode.")
-;;;###autoload
-(make-variable-buffer-local 'view-mode)
 
 (defcustom view-mode-hook nil
   "Normal hook run when starting to view a buffer or file."
-  :type 'hook
-  :group 'view)
+  :type 'hook)
 
-(defvar view-old-buffer-read-only nil)
-(make-variable-buffer-local 'view-old-buffer-read-only)
+(defvar-local view-old-buffer-read-only nil)
 
-(defvar view-old-Helper-return-blurb)
-(make-variable-buffer-local 'view-old-Helper-return-blurb)
+(defvar-local view-old-Helper-return-blurb nil)
 
-(defvar view-page-size nil
+(defvar-local view-page-size nil
   "Default number of lines to scroll by View page commands.
 If nil that means use the window size.")
-(make-variable-buffer-local 'view-page-size)
 
-(defvar view-half-page-size nil
+(defvar-local view-half-page-size nil
   "Default number of lines to scroll by View half page commands.
 If nil that means use half the window size.")
-(make-variable-buffer-local 'view-half-page-size)
 
-(defvar view-last-regexp nil)
-(make-variable-buffer-local 'view-last-regexp) ; Global is better???
+(defvar-local view-last-regexp nil) ; Global is better???
 
-(defvar view-return-to-alist nil
+(defvar-local view-return-to-alist nil
   "What to do with used windows and where to go when finished viewing buffer.
 This is local in each buffer being viewed.
 It is added to by `view-mode-enter' when starting to view a buffer and
@@ -134,17 +122,16 @@ subtracted from by `view-mode-exit' when finished viewing the buffer.
 See RETURN-TO-ALIST argument of function `view-mode-exit' for the format of
 `view-return-to-alist'.")
 (make-obsolete-variable
- 'view-return-to-alist "this variable is no more used." "24.1")
-(make-variable-buffer-local 'view-return-to-alist)
+ 'view-return-to-alist "this variable is no longer used." "24.1")
 (put 'view-return-to-alist 'permanent-local t)
 
-(defvar view-exit-action nil
-  "If non-nil, a function with one argument (a buffer) called when finished viewing.
+(defvar-local view-exit-action nil
+  "If non-nil, a function called when finished viewing.
+The function should take one argument (a buffer).
 Commands like \\[view-file] and \\[view-file-other-window] may
 set this to bury or kill the viewed buffer.
 Observe that the buffer viewed might not appear in any window at
 the time this function is called.")
-(make-variable-buffer-local 'view-exit-action)
 
 (defvar view-no-disable-on-exit nil
   "If non-nil, View mode \"exit\" commands don't actually disable View mode.
@@ -152,71 +139,70 @@ Instead, these commands just switch buffers or windows.
 This is set in certain buffers by specialized features such as help commands
 that use View mode automatically.")
 
-(defvar view-overlay nil
+(defvar-local view-overlay nil
   "Overlay used to display where a search operation found its match.
 This is local in each buffer, once it is used.")
-(make-variable-buffer-local 'view-overlay)
 
 ;; Define keymap inside defvar to make it easier to load changes.
 ;; Some redundant "less"-like key bindings below have been commented out.
 (defvar view-mode-map
   (let ((map (make-sparse-keymap)))
-    (define-key map "C" 'View-kill-and-leave)
-    (define-key map "c" 'View-leave)
-    (define-key map "Q" 'View-quit-all)
-    (define-key map "E" 'View-exit-and-edit)
-    ;; (define-key map "v" 'View-exit)
-    (define-key map "e" 'View-exit)
-    (define-key map "q" 'View-quit)
-    ;; (define-key map "N" 'View-search-last-regexp-backward)
-    (define-key map "p" 'View-search-last-regexp-backward)
-    (define-key map "n" 'View-search-last-regexp-forward)
-    ;; (define-key map "?" 'View-search-regexp-backward) ; Less does this.
-    (define-key map "\\" 'View-search-regexp-backward)
-    (define-key map "/" 'View-search-regexp-forward)
-    (define-key map "r" 'isearch-backward)
-    (define-key map "s" 'isearch-forward)
-    (define-key map "m" 'point-to-register)
-    (define-key map "'" 'register-to-point)
-    (define-key map "x" 'exchange-point-and-mark)
-    (define-key map "@" 'View-back-to-mark)
-    (define-key map "." 'set-mark-command)
-    (define-key map "%" 'View-goto-percent)
-    ;; (define-key map "G" 'View-goto-line-last)
-    (define-key map "g" 'View-goto-line)
-    (define-key map "=" 'what-line)
-    (define-key map "F" 'View-revert-buffer-scroll-page-forward)
-    ;; (define-key map "k" 'View-scroll-line-backward)
-    (define-key map "y" 'View-scroll-line-backward)
-    ;; (define-key map "j" 'View-scroll-line-forward)
-    (define-key map "\n" 'View-scroll-line-forward)
-    (define-key map "\r" 'View-scroll-line-forward)
-    (define-key map "u" 'View-scroll-half-page-backward)
-    (define-key map "d" 'View-scroll-half-page-forward)
-    (define-key map "z" 'View-scroll-page-forward-set-page-size)
-    (define-key map "w" 'View-scroll-page-backward-set-page-size)
-    ;; (define-key map "b" 'View-scroll-page-backward)
-    (define-key map "\C-?" 'View-scroll-page-backward)
-    ;; (define-key map "f" 'View-scroll-page-forward)
-    (define-key map " " 'View-scroll-page-forward)
-    (define-key map [?\S-\ ]  'View-scroll-page-backward)
-    (define-key map "o" 'View-scroll-to-buffer-end)
-    (define-key map ">" 'end-of-buffer)
-    (define-key map "<" 'beginning-of-buffer)
-    (define-key map "-" 'negative-argument)
-    (define-key map "9" 'digit-argument)
-    (define-key map "8" 'digit-argument)
-    (define-key map "7" 'digit-argument)
-    (define-key map "6" 'digit-argument)
-    (define-key map "5" 'digit-argument)
-    (define-key map "4" 'digit-argument)
-    (define-key map "3" 'digit-argument)
-    (define-key map "2" 'digit-argument)
-    (define-key map "1" 'digit-argument)
-    (define-key map "0" 'digit-argument)
-    (define-key map "H" 'describe-mode)
-    (define-key map "?" 'describe-mode)	; Maybe do as less instead? See above.
-    (define-key map "h" 'describe-mode)
+    (define-key map "C" #'View-kill-and-leave)
+    (define-key map "c" #'View-leave)
+    (define-key map "Q" #'View-quit-all)
+    (define-key map "E" #'View-exit-and-edit)
+    ;; (define-key map "v" #'View-exit)
+    (define-key map "e" #'View-exit)
+    (define-key map "q" #'View-quit)
+    ;; (define-key map "N" #'View-search-last-regexp-backward)
+    (define-key map "p" #'View-search-last-regexp-backward)
+    (define-key map "n" #'View-search-last-regexp-forward)
+    ;; (define-key map "?" #'View-search-regexp-backward) ; Less does this.
+    (define-key map "\\" #'View-search-regexp-backward)
+    (define-key map "/" #'View-search-regexp-forward)
+    (define-key map "r" #'isearch-backward)
+    (define-key map "s" #'isearch-forward)
+    (define-key map "m" #'point-to-register)
+    (define-key map "'" #'register-to-point)
+    (define-key map "x" #'exchange-point-and-mark)
+    (define-key map "@" #'View-back-to-mark)
+    (define-key map "." #'set-mark-command)
+    (define-key map "%" #'View-goto-percent)
+    ;; (define-key map "G" #'View-goto-line-last)
+    (define-key map "g" #'View-goto-line)
+    (define-key map "=" #'what-line)
+    (define-key map "F" #'View-revert-buffer-scroll-page-forward)
+    ;; (define-key map "k" #'View-scroll-line-backward)
+    (define-key map "y" #'View-scroll-line-backward)
+    ;; (define-key map "j" #'View-scroll-line-forward)
+    (define-key map "\n" #'View-scroll-line-forward)
+    (define-key map "\r" #'View-scroll-line-forward)
+    (define-key map "u" #'View-scroll-half-page-backward)
+    (define-key map "d" #'View-scroll-half-page-forward)
+    (define-key map "z" #'View-scroll-page-forward-set-page-size)
+    (define-key map "w" #'View-scroll-page-backward-set-page-size)
+    ;; (define-key map "b" #'View-scroll-page-backward)
+    (define-key map "\C-?" #'View-scroll-page-backward)
+    ;; (define-key map "f" #'View-scroll-page-forward)
+    (define-key map " " #'View-scroll-page-forward)
+    (define-key map [?\S-\ ] #'View-scroll-page-backward)
+    (define-key map "o" #'View-scroll-to-buffer-end)
+    (define-key map ">" #'end-of-buffer)
+    (define-key map "<" #'beginning-of-buffer)
+    (define-key map "-" #'negative-argument)
+    (define-key map "9" #'digit-argument)
+    (define-key map "8" #'digit-argument)
+    (define-key map "7" #'digit-argument)
+    (define-key map "6" #'digit-argument)
+    (define-key map "5" #'digit-argument)
+    (define-key map "4" #'digit-argument)
+    (define-key map "3" #'digit-argument)
+    (define-key map "2" #'digit-argument)
+    (define-key map "1" #'digit-argument)
+    (define-key map "0" #'digit-argument)
+    (define-key map "H" #'describe-mode)
+    (define-key map "?" #'describe-mode)	; Maybe do as less instead? See above.
+    (define-key map "h" #'describe-mode)
     map))
 
 ;;; Commands that enter or exit view mode.
@@ -227,7 +213,7 @@ This is local in each buffer, once it is used.")
 ;; types C-x C-q again to return to view mode.
 ;;;###autoload
 (defun kill-buffer-if-not-modified (buf)
-  "Like `kill-buffer', but does nothing if the buffer is modified."
+  "Like `kill-buffer', but does nothing if buffer BUF is modified."
   (let ((buf (get-buffer buf)))
     (and buf (not (buffer-modified-p buf))
 	 (kill-buffer buf))))
@@ -312,7 +298,7 @@ file: Users may suspend viewing in order to modify the buffer.
 Exiting View mode will then discard the user's edits.  Setting
 EXIT-ACTION to `kill-buffer-if-not-modified' avoids this.
 
-This function does not enable View mode if the buffer's major-mode
+This function does not enable View mode if the buffer's major mode
 has a `special' mode-class, because such modes usually have their
 own View-like bindings."
   (interactive "bView buffer: ")
@@ -338,7 +324,7 @@ Optional argument EXIT-ACTION is either nil or a function with buffer as
 argument.  This function is called when finished viewing buffer.  Use
 this argument instead of explicitly setting `view-exit-action'.
 
-This function does not enable View mode if the buffer's major-mode
+This function does not enable View mode if the buffer's major mode
 has a `special' mode-class, because such modes usually have their
 own View-like bindings."
   (interactive "bIn other window view buffer:\nP")
@@ -365,7 +351,7 @@ Optional argument EXIT-ACTION is either nil or a function with buffer as
 argument.  This function is called when finished viewing buffer.  Use
 this argument instead of explicitly setting `view-exit-action'.
 
-This function does not enable View mode if the buffer's major-mode
+This function does not enable View mode if the buffer's major mode
 has a `special' mode-class, because such modes usually have their
 own View-like bindings."
   (interactive "bView buffer in other frame: \nP")
@@ -381,13 +367,10 @@ own View-like bindings."
   ;; bindings instead of using the \\[] construction.  The reason for this
   ;; is that most commands have more than one key binding.
   "Toggle View mode, a minor mode for viewing text but not editing it.
-With a prefix argument ARG, enable View mode if ARG is positive,
-and disable it otherwise.  If called from Lisp, enable View mode
-if ARG is omitted or nil.
 
 When View mode is enabled, commands that do not change the buffer
-contents are available as usual.  Kill commands insert text in
-kill buffers but do not delete.  Most other commands beep and
+contents are available as usual.  Kill commands save text but
+do not delete it from the buffer.  Most other commands beep and
 tell the user that the buffer is read-only.
 
 \\<view-mode-map>
@@ -562,7 +545,10 @@ This function runs the normal hook `view-mode-hook'."
 
   (unless view-mode
     (view-mode 1)
-    (unless view-inhibit-help-message
+    (when (and (not view-inhibit-help-message)
+               ;; Avoid spamming the echo area if `view-mode' is entered
+               ;; non-interactively, e.g., in a temporary buffer (bug#44629).
+               this-command)
       (message "%s"
 	       (substitute-command-keys "\
 View mode: type \\[help-command] for help, \\[describe-mode] for commands, \\[View-quit] to quit.")))))
@@ -584,7 +570,7 @@ the associations of any windows with the current buffer.
 EXIT-ACTION, if non-nil, must specify a function that is called
 with the current buffer as argument and is called after disabling
 `view-mode' and removing any associations of windows with the
-current buffer. "
+current buffer."
   (when view-mode
     (let ((buffer (window-buffer)))
       (unless view-no-disable-on-exit
@@ -669,8 +655,8 @@ previous state and go to previous buffer or window."
   (recenter '(1)))
 
 (defun view-page-size-default (lines)
-  ;; If LINES is nil, 0, or larger than `view-window-size', return nil.
-  ;; Otherwise, return LINES.
+  "If LINES is nil, 0, or larger than `view-window-size', return nil.
+Otherwise, return LINES."
   (and lines
        (not (zerop (setq lines (prefix-numeric-value lines))))
        (<= (abs lines)
@@ -678,7 +664,7 @@ previous state and go to previous buffer or window."
        (abs lines)))
 
 (defun view-set-half-page-size-default (lines)
-  ;; Get and maybe set half page size.
+  "Get and maybe set half page size."
   (if (not lines) (or view-half-page-size
 		      (/ (view-window-size) 2))
     (setq view-half-page-size
@@ -746,18 +732,19 @@ invocations return to earlier marks."
     (setq backward (not backward) lines (- lines)))
   (when (and maxdefault lines (> lines (view-window-size)))
     (setq lines nil))
-  (cond (backward (scroll-down lines))
+  (cond (backward (scroll-down-command lines))
 	((view-really-at-end)
 	 (if view-scroll-auto-exit
 	     (View-quit)
 	   (ding)
 	   (view-end-message)))
-	(t (scroll-up lines)
+	(t (scroll-up-command lines)
 	   (if (view-really-at-end) (view-end-message)))))
 
 (defun view-really-at-end ()
-  ;; Return true if buffer end visible.  Maybe revert buffer and test.
-  (and (pos-visible-in-window-p (point-max))
+  "Return non-nil if buffer end visible.  Maybe revert buffer and test."
+  (and (or (null scroll-error-top-bottom) (eobp))
+       (pos-visible-in-window-p (point-max))
        (let ((buf (current-buffer))
 	     (bufname (buffer-name))
 	     (file (buffer-file-name)))
@@ -778,7 +765,7 @@ invocations return to earlier marks."
 	       (pos-visible-in-window-p (point-max)))))))
 
 (defun view-end-message ()
-  ;; Tell that we are at end of buffer.
+  "Tell that we are at end of buffer."
   (goto-char (point-max))
   (if (window-parameter nil 'quit-restore)
       (message "End of buffer.  Type %s to quit viewing."
@@ -960,7 +947,7 @@ for highlighting the match that is found."
      (t (error "No previous View-mode search")))
     (save-excursion
       (if end (goto-char (if (< times 0) (point-max) (point-min)))
-	(move-to-window-line (if (< times 0) 0 -1)))
+	(forward-line (if (< times 0) -1 1)))
       (if (if no (view-search-no-match-lines times regexp)
 	    (re-search-forward regexp nil t times))
 	  (setq where (point))))
@@ -985,7 +972,7 @@ for highlighting the match that is found."
 ;; https://lists.gnu.org/r/bug-gnu-emacs/2007-09/msg00073.html
 (defun view-search-no-match-lines (times regexp)
   "Search for the TIMESth occurrence of a line with no match for REGEXP.
-If such a line is found, return non-nil and set the match-data to that line.
+If such a line is found, return non-nil and set the match data to that line.
 If TIMES is negative, search backwards."
   (let ((step (if (>= times 0) 1
                 (setq times (- times))
