@@ -1,5 +1,5 @@
 /* Cursor motion subroutines for GNU Emacs.
-   Copyright (C) 1985, 1995, 2001-2017 Free Software Foundation, Inc.
+   Copyright (C) 1985, 1995, 2001-2022 Free Software Foundation, Inc.
     based primarily on public domain code written by Chris Torek
 
 This file is part of GNU Emacs.
@@ -15,14 +15,14 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.  */
+along with GNU Emacs.  If not, see <https://www.gnu.org/licenses/>.  */
 
 
 #include <config.h>
-#include <stdio.h>
 
 #include "lisp.h"
 #include "cm.h"
+#include "sysstdio.h"
 #include "termchar.h"
 #include "tparam.h"
 
@@ -30,7 +30,6 @@ along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.  */
 
 int cost;		/* sums up costs */
 
-/* ARGSUSED */
 int
 evalcost (int c)
 {
@@ -205,7 +204,7 @@ calccost (struct tty_display_info *tty,
 	p = tty->Wcm->cm_down, c = tty->Wcm->cc_down;
     if (c == BIG) {		/* caint get thar from here */
 	if (doit)
-	    printf ("OOPS");
+	  fputs ("OOPS", stdout);
 	return c;
     }
     totalcost = c * deltay;
@@ -288,7 +287,7 @@ dodelta:
     if (c == BIG) {		/* caint get thar from here */
 fail:
 	if (doit)
-	    printf ("OOPS");
+	  fputs ("OOPS", stdout);
 	return BIG;
     }
     totalcost += c * deltax;
@@ -321,7 +320,7 @@ cmgoto (struct tty_display_info *tty, int row, int col)
             llcost,
             relcost,
             directcost;
-    int     use IF_LINT (= 0);
+    int use UNINIT;
     char *p;
     const char *dcm;
 
